# Copyright 2021 (c) Anna Schumaker.
import lib
import pathlib
import tagdb
import unittest
from gi.repository import GLib
from gi.repository import Gst
from . import bass
from . import player
from . import scale
from . import selector

main_context = GLib.main_context_default()
test_album = pathlib.Path("./data/Test Album/")
test_track = test_album / "01 - Test Track.ogg"
test_uri = test_track.absolute().as_uri()


class FakeSelector(selector.Selector):
    def __init__(self, library):
        selector.Selector.__init__(self)
        t1 = [ t for t in library.tracks if t.tracknumber == 1 ][0]
        t2 = [ t for t in library.tracks if t.tracknumber == 11 ][0]
        self.tracks = [ t1, t2 ]
        self.cur = -1

    def next(self):
        self.cur += 1
        return self.tracks[self.cur]

    def previous(self):
        self.cur -= 1
        return self.tracks[self.cur]



class TestPlayer(unittest.TestCase):
    def setUpClass():
        tagdb.reset()
        lib = tagdb.Library.add(test_album)
        lib.scan().join()

    def setUp(self):
        self.changed = None
        lib.settings.reset()
        self.library = tagdb.Library.store[test_album]
        self.track = [ t for t in self.library.tracks if t.tracknumber == 1 ][0]

    def tearDownClass():
        tagdb.reset()

    def on_track_changed(self, player, prev, new):
        self.changed = (prev, new)

    def test_player_init(self):
        play = player.Player()
        self.assertIsInstance(play, bass.BassPlayer)
        self.assertIsInstance(play.bus, Gst.Bus)
        self.assertIsInstance(play.Autopause, scale.AutoPauseScale)
        self.assertIsInstance(play.Selector, selector.Selector)
        self.assertIsNone(play.track)

    def test_player_load_track(self):
        play = player.Player()
        play.connect("track-changed", self.on_track_changed)

        self.assertEqual(play.get_property("uri"), None)

        play.load_track(self.track)
        self.assertEqual(play.track, self.track)
        self.assertEqual(lib.settings.get_int("audio.trackid"), self.track.trackid)
        self.assertEqual(self.changed, (None, self.track) )
        self.assertTrue(play.playing)

        tagdb.tags.User["Previous"].tracks.clear()
        play2 = player.Player()
        self.assertEqual(play2.track, self.track)
        self.assertIn(self.track, tagdb.tags.User["Previous"].tracks)

    def test_player_play_pause(self):
        play = player.Player()

        play.load_track(self.track)
        self.assertEqual(play.track, self.track)

        play.pause()
        self.assertFalse(play.playing)

        play.play()
        self.assertTrue(play.playing)

        play.playpause()
        self.assertFalse(play.playing)

        play.playpause()
        self.assertTrue(play.playing)

        play.pause()
        self.assertFalse(play.playing)

    def test_player_next_previous(self):
        select = FakeSelector(self.library)
        play = player.Player()
        play.connect("track-changed", self.on_track_changed)
        play.Selector = select

        play.next()
        self.assertEqual(play.track.trackid, 1)
        self.assertEqual(self.changed, (None, tagdb.Tracks[1]) )

        play.next()
        self.assertEqual(play.track.trackid, 11)
        self.assertEqual(self.changed, (tagdb.Tracks[1], tagdb.Tracks[11]) )

        play.previous()
        self.assertEqual(play.track.trackid, 1)
        self.assertEqual(self.changed, (tagdb.Tracks[11], tagdb.Tracks[1]) )
