# Copyright 2021 (c) Anna Schumaker.
import db
import sqlite3
import unittest
from gi.repository import GObject

class TestAlbum(unittest.TestCase):
    def track_added(self, album, added):
        self.added = added

    def track_removed(self, album, removed):
        self.removed = removed

    def setUp(self):
        db.reset()

    def test_init(self):
        artist = db.artist.Table.find("Test Artist", "Test Sort")
        album = artist.find_album("Test Album")
        self.assertIsInstance(album, db.playlist.ParentPlaylist)
        self.assertEqual(album.get_property("name"), "Test Album")
        self.assertEqual(album.get_property("icon-name"), "media-optical-cd-audio")
        self.assertEqual(album.get_child_table(), db.disc.Table)

    def test_delete(self):
        artist = db.artist.Table.find("Test Artist", "Test Sort")
        album = artist.find_album("Test Album")
        album.delete()
        self.assertIsNone(db.album.Table.lookup(artist, "Test Album"))

    def test_find_disc(self):
        artist = db.artist.Table.find("Test Artist", "Test Sort")
        album = artist.find_album("Test Album")
        disc = album.find_disc(1, None)
        self.assertIsInstance(disc, db.disc.Disc)

    def test_tracks(self):
        artist = db.artist.Table.find("Test Artist", "Test Sort")
        album = artist.find_album("Test Album")
        album.connect("track-added", self.track_added)
        self.assertEqual(album.get_n_tracks(), 0)
        self.assertEqual(album.get_tracks(), [ ])

        track = db.make_fake_track(1, 1, "Test Track", "/a/b/c/1.ogg")
        self.assertEqual(album.get_n_tracks(), 1)
        self.assertEqual(album.get_track(0), track)
        self.assertEqual(album.get_tracks(), [ track ])
        self.assertEqual(album.get_track_index(track), 0)
        self.assertEqual(self.added, track)

        album.connect("track-removed", self.track_removed)
        db.track.Table.delete(track)
        self.assertEqual(album.get_tracks(), [ ])
        self.assertEqual(self.removed, track)


class TestAlbumTable(unittest.TestCase):
    def setUp(self):
        db.reset()

    def test_init(self):
        table = db.album.AlbumTable()
        self.assertIsInstance(table, db.playlist.ChildModel)
        self.assertEqual(table.table, "albums")
        self.assertEqual(table.parent, "artistid")
        self.assertEqual(table.order, "sort")

        self.assertIsInstance(db.album.Table, db.album.AlbumTable)
        db.sql.execute("SELECT albumid,artistid,plstateid,name,sort FROM albums")

    def test_insert(self):
        artist = db.artist.Table.insert("Test Artist", "Test Sort")
        album  = artist.find_album("Test Album")

        self.assertIsInstance(album, db.album.Album)
        self.assertEqual(album._name, "Test Album")
        self.assertEqual(album._rowkey, "albumid")

        with self.assertRaises(sqlite3.IntegrityError):
            db.album.Table.insert(artist, "Test Album")

    def test_lookup(self):
        artist = db.artist.Table.insert("Test Artist", "Test Sort")
        album  = artist.find_album("Test Album")
        self.assertEqual(db.album.Table.lookup(artist, "Test Album"), album)
        self.assertIsNone(db.album.Table.lookup(artist, "none"))
