# Copyright 2021 (c) Anna Schumaker.
import db
import sqlite3
import unittest
from gi.repository import GObject

class TestArtist(unittest.TestCase):
    def track_added(self, artist, added):
        self.added = added

    def track_removed(self, artist, removed):
        self.removed = removed

    def setUp(self):
        db.reset()

    def test_init(self):
        artist = db.artist.Table.find("Test Artist", "Test Sort")
        self.assertIsInstance(artist, db.playlist.ParentPlaylist)
        self.assertEqual(artist.get_property("name"), "Test Artist")
        self.assertEqual(artist.get_property("icon-name"), "avatar-default-symbolic")
        self.assertEqual(artist.get_child_table(), db.album.Table)

    def test_delete(self):
        artist = db.artist.Table.find("Test Artist", "Test Sort")
        artist.delete()
        self.assertIsNone(db.artist.Table.lookup("Test Artist"))

    def test_find_album(self):
        artist = db.artist.Table.find("Test Artist", "Test Sort")
        album = artist.find_album("Test Album")
        self.assertIsInstance(album, db.album.Album)

    def test_tracks(self):
        artist = db.artist.Table.find("Test Artist", "Test Sort")
        artist.connect("track-added", self.track_added)
        self.assertEqual(artist.get_n_tracks(), 0)
        self.assertEqual(artist.get_tracks(), [ ])

        track = db.make_fake_track(1, 1, "Test Track", "/a/b/c/1.ogg")
        self.assertEqual(artist.get_n_tracks(), 1)
        self.assertEqual(artist.get_track(0), track)
        self.assertEqual(artist.get_tracks(), [ track ])
        self.assertEqual(artist.get_track_index(track), 0)
        self.assertEqual(self.added, track)

        artist.connect("track-removed", self.track_removed)
        db.track.Table.delete(track)
        self.assertEqual(artist.get_tracks(), [ ])
        self.assertEqual(self.removed, track)


class TestArtistTable(unittest.TestCase):
    def setUp(self):
        db.reset()

    def test_init(self):
        table = db.artist.ArtistTable()
        self.assertIsInstance(table, db.playlist.Model)
        self.assertEqual(table.table, "artists")
        self.assertEqual(table.order, "sort")

        self.assertIsInstance(db.artist.Table, db.artist.ArtistTable)
        db.sql.execute("SELECT artistid,plstateid,name,sort FROM artists")

    def test_insert(self):
        table = db.artist.ArtistTable()
        artist = table.insert("Test Artist", "Test Sort")

        self.assertIsInstance(artist, db.artist.Artist)
        self.assertEqual(artist._name, "Test Artist")
        self.assertEqual(artist._rowkey, "artistid")

        with self.assertRaises(sqlite3.IntegrityError):
            db.artist.Table.insert("Test Artist", "Test Sort")

    def test_lookup(self):
        table = db.artist.ArtistTable()
        artist = table.insert("Test Artist", "Test Sort")
        self.assertEqual(table.lookup("Test Artist"), artist)
        self.assertIsNone(table.lookup("none"))
