# Copyright 2021 (c) Anna Schumaker.
import db
import sqlite3
import unittest
from gi.repository import GObject

class TestDecade(unittest.TestCase):
    def children_changed(self, decade, pos, rm, add):
        self.changed = (pos, rm, add)

    def track_added(self, decade, added):
        self.added = added

    def track_removed(self, decade, removed):
        self.removed = removed

    def setUp(self):
        db.reset()

    def test_init(self):
        decade = db.decade.Table.insert(2020)
        self.assertIsInstance(decade, db.playlist.ParentPlaylist)
        self.assertEqual(decade.get_property("name"), "2020s")
        self.assertEqual(decade.get_property("icon-name"), "x-office-calendar")
        self.assertEqual(decade.get_child_table(), db.year.Table)

    def test_decade(self):
        decade = db.decade.Table.insert(2020)
        self.assertEqual(decade._decade, 2020)
        self.assertEqual(decade.get_property("decade"), 2020)

    def test_delete(self):
        decade = db.decade.Table.find(2020)
        decade.delete()
        self.assertIsNone(db.decade.Table.lookup(2020))

    def test_find_year(self):
        decade = db.decade.Table.insert(2020)
        decade.connect("children-changed", self.children_changed)

        year = decade.find_year(2021)
        self.assertIsInstance(year, db.year.Year)
        self.assertEqual(self.changed, (0, 0, 1))

    def test_tracks(self):
        decade = db.decade.Table.insert(2020)
        decade.connect("track-added", self.track_added)
        self.assertEqual(decade.get_n_tracks(), 0)
        self.assertEqual(decade.get_tracks(), [ ])

        track = db.make_fake_track(1, 1, "Test Track", "/a/b/c/1.ogg")
        self.assertEqual(decade.get_n_tracks(), 1)
        self.assertEqual(decade.get_track(0), track)
        self.assertEqual(decade.get_tracks(), [ track ])
        self.assertEqual(decade.get_track_index(track), 0)
        self.assertEqual(self.added, track)

        decade.connect("track-removed", self.track_removed)
        db.track.Table.delete(track)
        self.assertEqual(decade.get_tracks(), [ ])
        self.assertEqual(self.removed, track)


class TestDecadeTable(unittest.TestCase):
    def setUp(self):
        db.reset()

    def test_init(self):
        table = db.decade.DecadeTable()
        self.assertIsInstance(table, db.playlist.Model)
        self.assertEqual(table.table, "decades")
        self.assertEqual(table.order, "decade")

        self.assertIsInstance(db.decade.Table, db.decade.DecadeTable)
        db.sql.execute("SELECT decadeid,plstateid,decade FROM decades")

    def test_insert(self):
        table = db.decade.DecadeTable()
        decade = table.insert(2020)

        self.assertIsInstance(decade, db.decade.Decade)
        self.assertEqual(decade._decade, 2020)
        self.assertEqual(decade._rowkey, "decadeid")

        with self.assertRaises(sqlite3.IntegrityError):
            db.decade.Table.insert(2020)

    def test_lookup(self):
        table = db.decade.DecadeTable()
        decade = table.insert(2020)

        self.assertEqual(table.lookup(2020), decade)
        self.assertIsNone(table.lookup(2021))
