# Copyright 2021 (c) Anna Schumaker.
import db
import sqlite3
import unittest
from gi.repository import GObject

class TestDisc(unittest.TestCase):
    def track_added(self, disc, added):
        self.added = added

    def track_removed(self, disc, removed):
        self. removed = removed

    def setUp(self):
        db.reset()

    def make_disc(self, artist, album, discno, subtitle):
        artist = db.artist.Table.find(artist, artist)
        return artist.find_album(album).find_disc(discno, subtitle)

    def test_init(self):
        disc = self.make_disc("Test Artist", "Test Album", 1, "")
        self.assertIsInstance(disc, db.playlist.Playlist)
        self.assertEqual(disc.get_property("icon-name"), "media-optical")
        self.assertEqual(disc._number, 1)

    def test_delete(self):
        artist = db.artist.Table.find("Test Artist", "Test Artist")
        album = artist.find_album("Test Album")
        disc = album.find_disc(1, "")
        disc.delete()
        self.assertIsNone(db.disc.Table.lookup(album, 1))

    def test_subtitle(self):
        disc = self.make_disc("Test Artist", "Test Album", 1, "Test Subtitle")
        self.assertEqual(disc._subtitle, "Test Subtitle")
        self.assertEqual(disc.get_property("subtitle"), "Test Subtitle")
        self.assertEqual(disc.get_property("name"), "1: Test Subtitle")

    def test_subtitle_none(self):
        disc = self.make_disc("Test Artist", "Test Album", 1, None)
        self.assertEqual(disc._subtitle, None)
        self.assertEqual(disc.get_property("subtitle"), "")
        self.assertEqual(disc.get_property("name"), "Disc 1")

    def test_subtitle_len_0(self):
        disc = self.make_disc("Test Artist", "Test Album", 1, "")
        self.assertEqual(disc._subtitle, "")
        self.assertEqual(disc.get_property("subtitle"), "")
        self.assertEqual(disc.get_property("name"), "Disc 1")

    def test_tracks(self):
        disc = self.make_disc("Test Artist", "Test Album", 1, "Test Subtitle")
        disc.connect("track-added", self.track_added)
        self.assertEqual(disc.get_n_tracks(), 0)
        self.assertEqual(disc.get_tracks(), [ ])

        track = db.make_fake_track(1, 1, "Test Track", "/a/b/c/1.ogg")
        self.assertEqual(disc.get_n_tracks(), 1)
        self.assertEqual(disc.get_track(0), track)
        self.assertEqual(disc.get_tracks(), [ track ])
        self.assertEqual(disc.get_track_index(track), 0)
        self.assertEqual(self.added, track)

        disc.connect("track-removed", self.track_removed)
        db.track.Table.delete(track)
        self.assertEqual(disc.get_tracks(), [ ])
        self.assertEqual(self.removed, track)


class TestDiscTable(unittest.TestCase):
    def setUp(self):
        db.reset()

    def test_init(self):
        table = db.disc.DiscTable()
        self.assertIsInstance(table, db.playlist.ChildModel)
        self.assertEqual(table.table, "discs")
        self.assertEqual(table.parent, "albumid")
        self.assertEqual(table.order, "number")

        self.assertIsInstance(db.disc.Table, db.disc.DiscTable)
        db.sql.execute("SELECT discid,albumid,plstateid,number,subtitle FROM discs")

    def test_insert(self):
        artist = db.artist.Table.insert("Test Artist", "Test Sort")
        album  = artist.find_album("Test Album")
        disc   = db.disc.Table.insert(album, 1, "subtitle")

        self.assertIsInstance(disc, db.disc.Disc)
        self.assertEqual(disc._number, 1)
        self.assertEqual(disc._subtitle, "subtitle")
        self.assertEqual(disc._rowkey, "discid")

        with self.assertRaises(sqlite3.IntegrityError):
            db.disc.Table.insert(album, 1, "subtitle")

    def test_lookup(self):
        artist = db.artist.Table.insert("Test Artist", "Test Sort")
        album  = artist.find_album("Test Album")
        disc   = album.find_disc(1, None)
        self.assertEqual(db.disc.Table.lookup(album, 1), disc)
        self.assertIsNone(db.disc.Table.lookup(album, "none"))
