# Copyright 2021 (c) Anna Schumaker.
import db
import sqlite3
import unittest
from gi.repository import GObject
from . import sql


class TestGenre(unittest.TestCase):
    def track_added(self, genre, added):
        self.added = added

    def track_removed(self, genre, removed):
        self.removed = removed

    def setUp(self):
        db.reset()

    def test_init(self):
        genre = db.genre.Table.find("Test Genre")
        self.assertIsInstance(genre, db.playlist.MappedPlaylist)
        self.assertEqual(genre._name, "Test Genre")
        self.assertEqual(genre.get_property("name"), "Test Genre")
        self.assertEqual(genre.get_property("icon-name"), "emblem-generic")
        self.assertEqual(genre.get_property("map-table"), "genre_map")

    def test_delete(self):
        track = db.make_fake_track(1, 1, "Test Track", "/a/b/c/1.ogg")
        genre = db.genre.Table.find("Test Genre")
        genre.add_track(track)
        genre.delete()
        self.assertEqual(genre.get_n_tracks(), 0)
        self.assertIsNone(db.genre.Table.lookup("Test Genre"))

    def test_add_remove_track(self):
        genre = db.genre.Table.find("Test Genre")
        genre.connect("track-added", self.track_added)
        self.assertEqual(genre.get_n_tracks(), 0)
        self.assertEqual(genre.get_tracks(), [ ])

        track = db.make_fake_track(1, 1, "Test Track", "/a/b/c/1.ogg")
        self.assertTrue(genre.add_track(track))
        self.assertEqual(genre.get_n_tracks(), 1)
        self.assertEqual(genre.get_track(0), track)
        self.assertEqual(genre.get_tracks(), [ track ])
        self.assertEqual(genre.get_track_index(track), 0)
        self.assertEqual(self.added, track)

        genre.connect("track-removed", self.track_removed)
        self.assertTrue(genre.remove_track(track))
        self.assertFalse(genre.remove_track(track))
        self.assertEqual(genre.get_n_tracks(), 0)
        self.assertEqual(genre.get_tracks(), [ ])
        self.assertEqual(self.removed, track)


class TestGenreTable(unittest.TestCase):
    def setUp(self):
        db.reset()

    def test_init(self):
        table = db.genre.GenreTable()
        self.assertIsInstance(table, db.playlist.Model)
        self.assertEqual(table.table, "genres")
        self.assertEqual(table.order, "sort")

        self.assertIsInstance(db.genre.Table, db.genre.GenreTable)
        db.sql.execute("SELECT genreid,plstateid,name,sort FROM genres")
        db.sql.execute("SELECT genreid,trackid FROM genre_map")

    def test_insert(self):
        table = db.genre.GenreTable()
        genre = table.insert("Test Genre")

        self.assertIsInstance(genre, db.genre.Genre)
        self.assertEqual(genre._name, "Test Genre")
        self.assertEqual(genre._rowkey, "genreid")

        with self.assertRaises(sqlite3.IntegrityError):
            db.genre.Table.insert("Test Genre")

    def test_lookup(self):
        genre = db.genre.Table.insert("Test Genre")
        self.assertEqual(db.genre.Table.lookup("Test Genre"), genre)
        self.assertIsNone(db.genre.Table.lookup("none"))
